@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2001, 2002, 2003, 2004 The SCons Foundation
+# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -28,15 +28,21 @@ and user errors in SCons.
 
 """
 
-__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Errors.py 0.96.93.D001 2006/11/06 08:31:54 knight"
+__revision__ = "src/engine/SCons/Errors.py 3057 2008/06/09 22:21:00 knight"
 
 
 
 class BuildError(Exception):
-    def __init__(self, node=None, errstr="Unknown error", filename=None, *args):
+    def __init__(self, node=None, errstr="Unknown error", status=0,
+                       filename=None, executor=None, action=None, command=None,
+                       *args):
         self.node = node
         self.errstr = errstr
+        self.status = status
         self.filename = filename
+        self.executor = executor
+        self.action = action
+        self.command = command
         apply(Exception.__init__, (self,) + args)
 
 class InternalError(Exception):
@@ -48,6 +54,9 @@ class UserError(Exception):
 class StopError(Exception):
     pass
 
+class EnvironmentError(Exception):
+    pass
+
 class ExplicitExit(Exception):
     def __init__(self, node=None, status=None, *args):
         self.node = node
