@@ -8,7 +8,7 @@ This creates a hash of global Aliases (dummy targets).
 """
 
 #
-# Copyright (c) 2001, 2002, 2003, 2004 The SCons Foundation
+# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -30,7 +30,7 @@ This creates a hash of global Aliases (dummy targets).
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Node/Alias.py 0.96.93.D001 2006/11/06 08:31:54 knight"
+__revision__ = "src/engine/SCons/Node/Alias.py 3057 2008/06/09 22:21:00 knight"
 
 import string
 import UserDict
@@ -57,10 +57,13 @@ class AliasNameSpace(UserDict.UserDict):
             return None
 
 class AliasNodeInfo(SCons.Node.NodeInfoBase):
-    pass
+    current_version_id = 1
+    field_list = ['csig']
+    def str_to_node(self, s):
+        return default_ans.Alias(s)
 
 class AliasBuildInfo(SCons.Node.BuildInfoBase):
-    pass
+    current_version_id = 1
 
 class Alias(SCons.Node.Node):
 
@@ -71,11 +74,17 @@ class Alias(SCons.Node.Node):
         SCons.Node.Node.__init__(self)
         self.name = name
 
+    def str_for_display(self):
+        return '"' + self.__str__() + '"'
+
     def __str__(self):
         return self.name
 
+    def make_ready(self):
+        self.get_csig()
+
     really_build = SCons.Node.Node.build
-    current = SCons.Node.Node.children_are_up_to_date
+    is_up_to_date = SCons.Node.Node.children_are_up_to_date
 
     def is_under(self, dir):
         # Make Alias nodes get built regardless of
@@ -85,9 +94,9 @@ class Alias(SCons.Node.Node):
 
     def get_contents(self):
         """The contents of an alias is the concatenation
-        of all the contents of its sources"""
-        contents = map(lambda n: n.get_contents(), self.children())
-        return string.join(contents, '')
+        of the content signatures of all its sources."""
+        childsigs = map(lambda n: n.get_csig(), self.children())
+        return string.join(childsigs, '')
 
     def sconsign(self):
         """An Alias is not recorded in .sconsign files"""
@@ -97,6 +106,13 @@ class Alias(SCons.Node.Node):
     #
     #
 
+    def changed_since_last_build(self, target, prev_ni):
+        cur_csig = self.get_csig()
+        try:
+            return cur_csig != prev_ni.csig
+        except AttributeError:
+            return 1
+
     def build(self):
         """A "builder" for aliases."""
         pass
@@ -107,6 +123,25 @@ class Alias(SCons.Node.Node):
         self.reset_executor()
         self.build = self.really_build
 
+    def get_csig(self):
+        """
+        Generate a node's content signature, the digested signature
+        of its content.
+
+        node - the node
+        cache - alternate node to use for the signature cache
+        returns - the content signature
+        """
+        try:
+            return self.ninfo.csig
+        except AttributeError:
+            pass
+
+        contents = self.get_contents()
+        csig = SCons.Util.MD5signature(contents)
+        self.get_ninfo().csig = csig
+        return csig
+
 default_ans = AliasNameSpace()
 
 SCons.Node.arg2nodes_lookups.append(default_ans.lookup)
