@@ -14,14 +14,15 @@ use absolute paths.  To hack around it, add '#/blah'.  This will link
 blah.lib from the directory where SConstruct resides.
 
 Compiler variables:
-    DC - The name of the D compiler to use.  Defaults to dmd.
+    DC - The name of the D compiler to use.  Defaults to dmd or gdmd,
+    whichever is found.
     DPATH - List of paths to search for import modules.
     DVERSIONS - List of version tags to enable when compiling.
     DDEBUG - List of debug tags to enable when compiling.
 
 Linker related variables:
     LIBS - List of library files to link in.
-    DLINK - Name of the linker to use.  Defaults to dmd.
+    DLINK - Name of the linker to use.  Defaults to dmd or gdmd.
     DLINKFLAGS - List of linker flags.
 
 Lib tool variables:
@@ -31,7 +32,7 @@ Lib tool variables:
 """
 
 #
-# Copyright (c) 2001, 2002, 2003, 2004 The SCons Foundation
+# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -53,7 +54,7 @@ Lib tool variables:
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Tool/dmd.py 0.96.93.D001 2006/11/06 08:31:54 knight"
+__revision__ = "src/engine/SCons/Tool/dmd.py 3057 2008/06/09 22:21:00 knight"
 
 import os
 import string
@@ -93,7 +94,8 @@ def generate(env):
     static_obj.add_emitter('.d', SCons.Defaults.StaticObjectEmitter)
     shared_obj.add_emitter('.d', SCons.Defaults.SharedObjectEmitter)
 
-    env['DC'] = 'dmd'
+    dc = env.Detect(['dmd', 'gdmd'])
+    env['DC'] = dc
     env['DCOM'] = '$DC $_DINCFLAGS $_DVERFLAGS $_DDEBUGFLAGS $_DFLAGS -c -of$TARGET $SOURCES'
     env['_DINCFLAGS'] = '$( ${_concat(DINCPREFIX, DPATH, DINCSUFFIX, __env__, RDirs, TARGET, SOURCE)}  $)'
     env['_DVERFLAGS'] = '$( ${_concat(DVERPREFIX, DVERSIONS, DVERSUFFIX, __env__)}  $)'
@@ -105,14 +107,15 @@ def generate(env):
     env['DVERSIONS'] = []
     env['DDEBUG'] = []
 
-    # Add the path to the standard library.
-    # This is merely for the convenience of the dependency scanner.
-    dmd_path = env.WhereIs('dmd')
-    if dmd_path:
-        x = string.rindex(dmd_path, 'dmd')
-        phobosDir = dmd_path[:x] + '/../src/phobos'
-        if os.path.isdir(phobosDir):
-            env.Append(DPATH = [phobosDir])
+    if dc:
+        # Add the path to the standard library.
+        # This is merely for the convenience of the dependency scanner.
+        dmd_path = env.WhereIs(dc)
+        if dmd_path:
+            x = string.rindex(dmd_path, dc)
+            phobosDir = dmd_path[:x] + '/../src/phobos'
+            if os.path.isdir(phobosDir):
+                env.Append(DPATH = [phobosDir])
 
     env['DINCPREFIX'] = '-I'
     env['DINCSUFFIX'] = ''
@@ -191,20 +194,25 @@ def generate(env):
             env['SMART_LINKCOM'] = smart_link[linkcom]
         except KeyError:
             def _smartLink(source, target, env, for_signature,
-                           defaultLinker=linkcom):
+                           defaultLinker=linkcom, dc=dc):
                 if isD(source):
                     try:
                         libs = env['LIBS']
                     except KeyError:
                         libs = []
                     if 'phobos' not in libs:
-                        env.Append(LIBS = ['phobos'])
+                        if dc is 'dmd':
+                            env.Append(LIBS = ['phobos'])
+                        elif dc is 'gdmd':
+                            env.Append(LIBS = ['gphobos'])
                     if 'pthread' not in libs:
                         env.Append(LIBS = ['pthread'])
+                    if 'm' not in libs:
+                        env.Append(LIBS = ['m'])
                 return defaultLinker
             env['SMART_LINKCOM'] = smart_link[linkcom] = _smartLink
 
         env['LINKCOM'] = '$SMART_LINKCOM '
 
 def exists(env):
-    return env.Detect('dmd')
+    return env.Detect(['dmd', 'gdmd'])
