@@ -9,7 +9,7 @@ selection method.
 """
 
 #
-# Copyright (c) 2001, 2002, 2003, 2004 The SCons Foundation
+# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -31,7 +31,7 @@ selection method.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Tool/javah.py 0.96.93.D001 2006/11/06 08:31:54 knight"
+__revision__ = "src/engine/SCons/Tool/javah.py 3057 2008/06/09 22:21:00 knight"
 
 import os.path
 import string
@@ -60,6 +60,7 @@ def emit_java_headers(target, source, env):
             except AttributeError:
                 classdir = '.'
     classdir = env.Dir(classdir).rdir()
+
     if str(classdir) == '.':
         c_ = None
     else:
@@ -77,10 +78,13 @@ def emit_java_headers(target, source, env):
                 classname = classname[:-len(class_suffix)]
             classname = SCons.Tool.javac.classname(classname)
         s = src.rfile()
-        s.attributes.java_classdir = classdir
         s.attributes.java_classname = classname
         slist.append(s)
 
+    s = source[0].rfile()
+    if not hasattr(s.attributes, 'java_classdir'):
+        s.attributes.java_classdir = classdir
+
     if target[0].__class__ is SCons.Node.FS.File:
         tlist = target
     else:
@@ -106,22 +110,22 @@ def JavaHOutFlagGenerator(target, source, env, for_signature):
     except AttributeError:
         return '-o ' + str(t)
 
-JavaHAction = SCons.Action.Action('$JAVAHCOM', '$JAVAHCOMSTR')
-
-JavaHBuilder = SCons.Builder.Builder(action = JavaHAction,
-                     emitter = emit_java_headers,
-                     src_suffix = '$JAVACLASSSUFFIX',
-                     target_factory = SCons.Node.FS.Entry,
-                     source_factory = SCons.Node.FS.File)
+def getJavaHClassPath(env,target, source, for_signature):
+    path = "${SOURCE.attributes.java_classdir}"
+    if env.has_key('JAVACLASSPATH') and env['JAVACLASSPATH']:
+        path = SCons.Util.AppendPath(path, env['JAVACLASSPATH'])
+    return "-classpath %s" % (path)
 
 def generate(env):
     """Add Builders and construction variables for javah to an Environment."""
-    env['BUILDERS']['JavaH'] = JavaHBuilder
+    java_javah = SCons.Tool.CreateJavaHBuilder(env)
+    java_javah.emitter = emit_java_headers
 
     env['_JAVAHOUTFLAG']    = JavaHOutFlagGenerator
     env['JAVAH']            = 'javah'
     env['JAVAHFLAGS']       = SCons.Util.CLVar('')
-    env['JAVAHCOM']         = '$JAVAH $JAVAHFLAGS $_JAVAHOUTFLAG -classpath ${SOURCE.attributes.java_classdir} ${SOURCES.attributes.java_classname}'
+    env['_JAVAHCLASSPATH']  = getJavaHClassPath
+    env['JAVAHCOM']         = '$JAVAH $JAVAHFLAGS $_JAVAHOUTFLAG $_JAVAHCLASSPATH ${SOURCES.attributes.java_classname}'
     env['JAVACLASSSUFFIX']  = '.class'
 
 def exists(env):
